-----------------------------------------------------------------------------
--
--  DAP.VHDL
--
--	Top-Level-Entity of design. Combines and connects all Modules to a complete IO-System.
--
--	Author 	: Rene Bllhoff
--  Created : around Dec. 2006
--
--	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
--
-------------------------------------------------------------------------------
--
--  This source file may be used and distributed without
--  restriction provided that this copyright statement is not
--  removed from the file and that any derivative work contains
--  the original copyright notice and the associated disclaimer.
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--										
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public
--  License along with this source; if not, download it	from
--  http://www.gnu.org/licenses/gpl.html	
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity dap is
  port (-- main supply (clocks and reset)
				clk50m		: in  std_logic;
				res				: in  std_logic;

				-- spi interface for uc access
				spi_clk		: in  std_logic;
				spi_do		: out std_logic;
				spi_di		: in  std_logic;
				spi_cs		: in  std_logic;

				-- 2 i2s interfaces (commen bclk and lrclk) designed as slave !
				bclk		  : in  std_logic;
				lrclk			: in  std_logic;
				sdin1 		: in  std_logic;
				sdout1		: out std_logic;
				sdin2			: in  std_logic;
				sdout2		: out std_logic;

				-- LEDs & Switches of Spartan 3 On Board Hardware
				swin			: in  std_logic_vector ( 7 downto 0);
				pbtn			: in  std_logic_vector ( 2 downto 0);
				ledout		: out std_logic_vector ( 7 downto 0);	
				segout		: out std_logic_vector ( 7 downto 0);
				digout		: out std_logic_vector ( 3 downto 0);


			  -- SRAM-Access of Spartan 3 On Board Hardware
--			  sri_data 	: INOUT std_logic_vector(15 downto 0);      
--			  sri_addr 	: OUT std_logic_vector(17 downto 0);
--			  sri_cs1 	: OUT std_logic;
--			  sri_cs2 	: OUT std_logic;
--			  sri_oe 		: OUT std_logic;
--			  sri_ub 		: OUT std_logic;
--			  sri_lb 		: OUT std_logic;
--			  sri_we 		: OUT std_logic;

			  -- select data-dispatcher interface (RS232 or SPI)
			  if_sel		: in  std_logic;

				-- pins for routing rs232 signals from onboard-max232 to dispatcher or uc
				rx_in  		: in 	std_logic;
				tx_out 		: out std_logic;

				-- pins for routing rs232 signals to uc
				rx_uC 		: out std_logic;
				tx_uC  		: in 	std_logic
			);	
end dap;

architecture Behavioral of dap is


---------------------------------------------
--
-- SPI-Component for uC Interfacing
--
---------------------------------------------	

	COMPONENT spi_slave
	PORT(
		clk50m : IN  std_logic;
		res : IN  std_logic;        
		spi_cs : IN  std_logic;
		spi_clk : IN  std_logic;
		spi_di : IN  std_logic;
		spi_do : OUT std_logic;                        
		cs : OUT std_logic;
		dat2host : IN  std_logic_vector (7 downto 0);
		dat2dev : OUT std_logic_vector (7 downto 0);
		dat2dev_rcv : OUT std_logic
		);
	END COMPONENT;


---------------------------------------------
--
-- RS232-Component for PC Interfacing
--
---------------------------------------------	

	COMPONENT uart2dispatcher
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		rx_in : IN std_logic;
		dp_in : IN std_logic_vector(7 downto 0);          
		tx_out : OUT std_logic;
		dp_out : OUT std_logic_vector(7 downto 0);
		dp_trg : OUT std_logic;
		dp_sel : OUT std_logic
		);
	END COMPONENT;


---------------------------------------------
--
-- Dispatcher for Interfacing 2 Devices
--
---------------------------------------------	

	COMPONENT data_dispatcher_2
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		dp_din   : in std_logic_vector (7 downto 0);
		dp_dout  : out std_logic_vector (7 downto 0);
		dp_trg	 : in std_logic;
		dp_sel   : in std_logic;
		dev1_rdy : IN std_logic;
		dev1_din : IN std_logic_vector(7 downto 0);
		dev1_inf : IN std_logic_vector(15 downto 0);
		dev2_rdy : IN std_logic;
		dev2_din : IN std_logic_vector(7 downto 0);
		dev2_inf : IN std_logic_vector(15 downto 0);          
		sysreset : OUT std_logic;
		dev_dout : OUT std_logic_vector(7 downto 0);
		dev1_sel : OUT std_logic;
		dev1_trg : OUT std_logic;
		dev1_res : OUT std_logic;
		dev2_sel : OUT std_logic;
		dev2_trg : OUT std_logic;
		dev2_res : OUT std_logic
		);
	END COMPONENT;


---------------------------------------------
--
-- StdIO-Component for simple OnBoard Hardware 
-- of the Spartan 3 Development Board
--
---------------------------------------------	

	COMPONENT device_stdio_v2
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		fsm_select : IN std_logic;
		fsm_trigger : IN std_logic;
		fsm_reset : IN std_logic;
		fsm_din : IN std_logic_vector(7 downto 0);
		grab_7seg : IN std_logic;
		grab_dots : IN std_logic;
		grab_leds : IN std_logic;
		use_hex_in : IN std_logic;
		use_dots_in : IN std_logic;
		data16in : IN std_logic_vector(15 downto 0);
		data32in : IN std_logic_vector(31 downto 0);
		dots4in : IN std_logic_vector(3 downto 0);
		leds8in : IN std_logic_vector(7 downto 0);
		swin : IN std_logic_vector(7 downto 0);
		pbtn : IN std_logic_vector(2 downto 0);          
		fsm_ready : OUT std_logic;
		fsm_dout : OUT std_logic_vector(7 downto 0);
		fsm_inf : OUT std_logic_vector(15 downto 0);
		ledout : OUT std_logic_vector(7 downto 0);
		segout : OUT std_logic_vector(7 downto 0);
		digout : OUT std_logic_vector(3 downto 0)
		);
	END COMPONENT;


---------------------------------------------
--
-- Digital Audio Processor (DAP) Device
--
---------------------------------------------	

	COMPONENT dap_core
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		fsm_din : IN std_logic_vector(7 downto 0);
		fsm_trg : IN std_logic;
		fsm_sel : IN std_logic;
		fsm_dout : OUT std_logic_vector(7 downto 0);
		fsm_rdy : OUT std_logic;
		fsm_inf : OUT std_logic_vector(15 downto 0);


		-- SRAM-Access IO (Spartan 3 On Board  Hardware)
--		sri_data 	: INOUT std_logic_vector(15 downto 0);      
--		sri_addr	: OUT std_logic_vector(17 downto 0);
--		sri_cs1 	: OUT std_logic;
--		sri_cs2 	: OUT std_logic;
--		sri_oe 		: OUT std_logic;
--		sri_ub 		: OUT std_logic;
--		sri_lb 		: OUT std_logic;
--		sri_we 		: OUT std_logic;

		bclk : in std_logic;
		lrclk : in std_logic;
		sdin1 : in std_logic;
		sdout1 : out std_logic;
		sdin2 : in std_logic;
		sdout2 : out std_logic
		);
	END COMPONENT;


---------------------------------------------
--
-- 	Simple PS/2 Keyboard Interface Device (unconnected)
--
---------------------------------------------	

--	COMPONENT spi_sm_ps2kbd
--	PORT(
--    clk      	: in  	std_logic;
--    res     	: in  	std_logic;  
--
--    ps2c 			: inout std_logic;
--    ps2d 			: inout std_logic;
--
--		kbd_sel  	: in  	std_logic;
--		kbd_trg 	: in  	std_logic;
--		kbd_rdy		: out 	std_logic;
--		kbd_din  	: in  	std_logic_vector ( 7 downto 0);
--		kbd_dout 	: out 	std_logic_vector ( 7 downto 0);
--		kbd_inf	  : out 	std_logic_vector (15 downto 0)
--		);
--	END COMPONENT;


---------------------------------------------
--
-- VGA-Text-Device (with chargen,color,text-ram) (unconnected)
--
---------------------------------------------	

--	COMPONENT spi_sm_vga_core
--	PORT(
--		clk  : IN std_logic;
--		res : IN std_logic;
--		vga_sel : IN std_logic;
--		vga_trg : IN std_logic;
--		vga_din : IN std_logic_vector(7 downto 0);          
--		vga_rdy : OUT std_logic;
--		vga_dout : OUT std_logic_vector(7 downto 0);
--		vga_inf : OUT std_logic_vector(15 downto 0);
--		vga_h : OUT std_logic;
--		vga_v : OUT std_logic;
--		vga_r : OUT std_logic;
--		vga_g : OUT std_logic;
--		vga_b : OUT std_logic
--		);
--	END COMPONENT;


---------------------------------------------
--
-- 	Used Signals
--
---------------------------------------------	



	-- UART2Dispatcher Component Signals (-> Muxer)
	signal uart_dout :	std_logic_vector (7 downto 0);
	signal uart_din  :	std_logic_vector (7 downto 0);
	signal uart_trg  :	std_logic;
	signal uart_csel :	std_logic;

	-- SPI2Dispatcher Component Signals (-> Muxer)
	signal spi_dout :	std_logic_vector (7 downto 0);
	signal spi_din  :	std_logic_vector (7 downto 0);
	signal spi_trg  :	std_logic;
	signal spi_csel :	std_logic;

	-- Data Dispatcher Component Signals (-> Dispatcher)
	signal ddp_dout :	std_logic_vector (7 downto 0);
	signal ddp_din  :	std_logic_vector (7 downto 0);
	signal ddp_trg  :	std_logic;
	signal ddp_csel :	std_logic;

	-- RS232 Signals (from OnBoard -> Uart2Dispatcher)
	signal rx_dp		: std_logic;
	signal tx_dp		: std_logic;

	-- Data-Output from Dispatcher Component
	-- (connected to all used devices)
	signal dev_dout  : std_logic_vector ( 7 downto 0);
	
	-- Signals of Port 1 of the Dispatcher
	signal dev1_trg  : std_logic;
	signal dev1_rdy  : std_logic;
	signal dev1_sel  : std_logic;
	signal dev1_res  : std_logic;
	signal dev1_din  : std_logic_vector ( 7 downto 0);
	signal dev1_inf  : std_logic_vector (15 downto 0);

	-- Signals of Port 2 of the Dispatcher
	signal dev2_trg  : std_logic;
	signal dev2_rdy  : std_logic;
	signal dev2_sel  : std_logic;
	signal dev2_res  : std_logic;
	signal dev2_din  : std_logic_vector ( 7 downto 0);
	signal dev2_inf  : std_logic_vector (15 downto 0);

	-- Signals of Port 3 of the Dispatcher
--	signal dev3_trg  : std_logic;
--	signal dev3_rdy  : std_logic;
--	signal dev3_sel  : std_logic;
--	signal dev3_res  : std_logic;
--	signal dev3_din  : std_logic_vector ( 7 downto 0);
--	signal dev3_inf  : std_logic_vector (15 downto 0);

	-- Signals of Port 4 of the Dispatcher
--	signal dev4_trg  : std_logic;
--	signal dev4_rdy  : std_logic;
--	signal dev4_sel  : std_logic;
--	signal dev4_res  : std_logic;
--	signal dev4_din  : std_logic_vector ( 7 downto 0);
--	signal dev4_inf  : std_logic_vector (15 downto 0);


begin




--------------------------------------------------
--
-- Muxer of Data Dispatcher Interface (RS232/SPI)
--
--------------------------------------------------

	-- if if_sel is 0 then the RS232 is routed to the uart which feeds the data-dispatcher
	-- if if_sel is 1 then the RS232 is routed to the uc, and spi feeds the data-dispatcher

	-- Routing of Dispatcher Signals from/to SPI or UART-Interface
	process (clk50m,res)
	begin
		if clk50m'event and clk50m = '1' then
		  if if_sel = '0'	then
				ddp_csel <= uart_csel;	-- simply route the outputs of the uart2dispatcher to the data dispatcher
				ddp_dout <= uart_dout;
				ddp_trg  <= uart_trg;
			else
				ddp_csel <= spi_csel;		-- or route the outputs of the spi-slave to the data dispatcher
				ddp_dout <= spi_dout;
				ddp_trg  <= spi_trg;
			end if;

			-- common signals to both components (uart2dispatcher and spi_slave)
			spi_din		<= ddp_din;
			uart_din	<= ddp_din;

		end if;
	end process;

	-- Routing of RS232 Signals to uC or Dispatcher

	process (clk50m,res)
	begin
		if clk50m'event and clk50m = '1' then
		  if if_sel = '0'	then
				tx_out <= tx_DP;	  -- tx is controlled by the uart
			else	
				tx_out <= tx_uC;		-- tx is controlled by the uc
			end if;

			-- rx signal (from pc) to uart and uc 
			rx_uC <= rx_in;
			rx_DP <= rx_in;
		end if;
	end process;



---------------------------------------------
--
-- Instanciation of Components
--
---------------------------------------------	

-- SPI-Slave-Component 

	Inst_spi_slave : spi_slave port map (
    clk50m  			=> clk50m,
    res     			=> res,     

		-- SPI-Signals from uC
    spi_cs  			=> spi_cs,
    spi_clk 			=> spi_clk,
    spi_do  			=> spi_do,
    spi_di  			=> spi_di,

		-- To Data Dispatcher
    dat2host     	=> spi_din,
    dat2dev      	=> spi_dout,
		dat2dev_rcv  	=> spi_trg,
		cs						=> spi_csel
  );


-- UART Component

	Inst_uart2dispatcher: uart2dispatcher PORT MAP(
		clk 		=> clk50m,
		res 		=> res,

		-- From RS232-Muxer
		rx_in 	=> rx_dp,
		tx_out 	=> tx_dp,

		-- To Data Dispatcher
		dp_out 	=> uart_dout,
		dp_in 	=> uart_din,
		dp_trg 	=> uart_trg,
		dp_sel 	=> uart_csel
	);


	--		
	-- Data Dispatcher-Component (2 Devices)
	--
	Inst_data_dispatcher_2: data_dispatcher_2 PORT MAP(
		clk50m 		=> clk50m,
		res 			=> res,

		-- From Dispatcher-Component
		dp_din		=> ddp_dout,
		dp_dout		=> ddp_din,
		dp_trg		=> ddp_trg,
		dp_Sel		=> ddp_csel,


		-- Global Outputs
		sysreset 	=> open,
		dev_dout 	=> dev_dout,

		--		
		-- Port 1 connected to StdIO-Component
		--
		dev1_sel 	=> dev1_sel,
		dev1_rdy 	=> dev1_rdy,
		dev1_trg 	=> dev1_trg,
		dev1_res 	=> dev1_res,
		dev1_din 	=> dev1_din,
		dev1_inf 	=> dev1_inf,

		--		
		-- Port 2 connected to DAP-Component
		--
		dev2_sel 	=> dev2_sel,
		dev2_rdy 	=> dev2_rdy,
		dev2_trg 	=> dev2_trg,
		dev2_res 	=> open,
		dev2_din 	=> dev2_din,
		dev2_inf 	=> dev2_inf
	);

	-- StdIO-Compoment

	Inst_device_stdio_v2: device_stdio_v2 PORT MAP(
		clk50m 			=> clk50m,
		res 				=> res,

		-- SPI-Dispatcher Interface
		fsm_select 	=> dev1_sel,
		fsm_trigger => dev1_trg,
		fsm_ready 	=> dev1_rdy,
		fsm_reset 	=> dev1_res,
		fsm_din   	=> dev_dout,
		fsm_dout    => dev1_din,
		fsm_inf  		=> dev1_inf,

	  -- No Grabbing for Debug
		grab_7seg   => '0',
		grab_dots   => '0',
		grab_leds   => '0',
		use_hex_in	=> '0',
		use_dots_in	=> '0',

		data16in		=> (others => '0'),
		data32in		=> (others => '0'),
		dots4in			=> (others => '0'),
		leds8in			=> (others => '0'),

		
		-- IO-Connections to Onboard Hardware
		swin 				=> swin,
		pbtn 				=> pbtn,			 
		ledout 			=> ledout,		 
		segout 			=> segout,
		digout 			=> digout
	);


	-- DAP-Compoment

	Inst_dap_core: dap_core PORT MAP(
		clk 				=> clk50m,
		res 				=> res,

		-- SPI-Dispatcher Interface
		fsm_din 		=> dev_dout,
		fsm_dout		=> dev2_din,
		fsm_trg 		=> dev2_trg,
		fsm_rdy 		=> dev2_rdy,
		fsm_sel 		=> dev2_sel,
		fsm_inf 		=> dev2_inf,

		-- IO-Connections for 1 SRAM
--		sri_data 	=> sri_data,
--		sri_addr 	=> sri_addr,
--		sri_cs1 	=> sri_cs1,
--		sri_cs2 	=> sri_cs2,
--		sri_oe 		=> sri_oe,
--		sri_ub 		=> sri_ub,
--		sri_lb 		=> sri_lb,
--		sri_we 		=> sri_we,

		-- IO-Connections for 2 I2S Interfaces
		bclk 				=> bclk,
 		lrclk 			=> lrclk,
		sdin1 			=> sdin1,
		sdout1 			=> sdout1,
		sdin2 			=> sdin2,
		sdout2 			=> sdout2
	);


	-- Simple PS2 Keyboard-Compoment (not connected)

--	Inst_spi_sm_ps2kbd: spi_sm_ps2kbd PORT MAP(
--		clk    			=> clk50m,
--		res 				=> res,
--
--		-- SPI-Dispatcher Interface
--		kbd_sel    	=> dev4_sel,
--		kbd_trg     => dev4_trg,
--		kbd_rdy   	=> dev4_rdy,
--		kbd_din    	=> dev_dout,
--		kbd_dout    => dev4_din,
--		kbd_inf     => dev4_inf,
--
--		-- IO-Connections to Onboard Hardware
--		ps2c 				=> ps2c,
--		ps2d 				=> ps2d
--	);


	-- VGA-Text-Compoment (not connected)

--	Inst_spi_sm_vga_core: spi_sm_vga_core PORT MAP(
--		clk 			  => clk50m,
--		res 				=> res,
--
--		-- SPI-Dispatcher Interface
--		vga_sel 		=> dev2_sel,
--		vga_trg 		=> dev2_trg,
--		vga_rdy 		=> dev2_rdy,
--		vga_din 		=> dev_dout,
--		vga_dout 		=> dev2_din,
--		vga_inf 		=> dev2_inf,
--
--		-- IO-Connections for VGA-Interface
--		vga_h 			=> vga_h,
--		vga_v 			=> vga_v,
--		vga_r 			=> vga_r,
--		vga_g 			=> vga_g,
--		vga_b 			=> vga_b
--	);



end Behavioral;
