library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity monoflop is
    Port ( clk50m 	 : in 	std_logic;
           res 			 : in 	std_logic;
           ticks		 : in 	std_logic_vector(31 downto 0);
           trigger 	 : in 	std_logic;
           output 	 : out std_logic);
end monoflop;

architecture Behavioral of monoflop is

	type mfStates is (stWaitTrigger, stWaitTime);
	signal monoflop_state : mfStates;

	signal counter : std_logic_vector (31 downto 0);
	signal trigger_z1, trigger_z2, trigger_z3 : std_logic;

begin

	process (clk50m, res, trigger_z1, trigger_z2, trigger_z3, counter, ticks, monoflop_state)
	begin
	  if res = '1' then
		  counter 				<= (others => '0');
		  output  				<= '0';
			monoflop_state 	<= stWaitTrigger;
			trigger_z1 			<= '0';
			trigger_z2 			<= '0';
			trigger_z3 			<= '0';

		elsif clk50m'event and clk50m = '1' then
			trigger_z1 <= trigger;
			trigger_z2 <= trigger_z1;
			trigger_z3 <= trigger_z2;

			case monoflop_state is
				when stWaitTrigger 	=> if trigger_z2 = '1' and trigger_z3 = '0' then
																 counter  <= counter + '1';
																 monoflop_state <= stWaitTime;
															 end if;	
				when stWaitTime 		=> if counter >= ticks then
																 counter	<= (others => '0');
																 monoflop_state <= stWaitTrigger;
																 output 	<= '0';
															 else
															   counter  <= counter + '1';
																 output 	<= '1';
															 end if;	
				when others 				=> NULL;
			end case;

		end if;

	end process;


end Behavioral;
