-----------------------------------------------------------------------------
--
--	DAP_CORE.VHDL
--
-- 	Digital Audio Processor (DAP) as Top-Level Entity.
--  Holds SDSP-Core and Util-Core as well as the Dispather-Interfae (to connect to
--  BlockRAMs, and IO; Mute and Usage), the 2x I2S Interface, Program-Memory and Main-Statemachine
--
--	Author 	: Rene Bllhoff
--  Created : around Apr. 2006
--
--	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
-----------------------------------------------------------------------------
--
--  This source file may be used and distributed without
--  restriction provided that this copyright statement is not
--  removed from the file and that any derivative work contains
--  the original copyright notice and the associated disclaimer.
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--										
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public
--  License along with this source; if not, download it	from
--  http://www.gnu.org/licenses/gpl.html	
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity dap_core is
    Port (-- main supply (clocks and reset)
					clk 			: in  std_logic;
          res 			: in  std_logic;

					-- Data-Dispatcher Interface
				  fsm_din		: in  std_logic_vector (7 downto 0);
				  fsm_dout	: out std_logic_vector (7 downto 0);
				  fsm_trg		: in  std_logic;
				  fsm_rdy		: out std_logic; 
				  fsm_sel		: in  std_logic;
				  fsm_inf		: out std_logic_vector (15 downto 0);


					-- SRAM-Access IO (Spartan 3 On Board Hardware)
					--sri_data 	: INOUT std_logic_vector(15 downto 0);      
					--sri_addr 	: OUT std_logic_vector(17 downto 0);
					--sri_cs1 	: OUT std_logic;
					--sri_cs2 	: OUT std_logic;
					--sri_oe 		: OUT std_logic;
					--sri_ub 		: OUT std_logic;
					--sri_lb 		: OUT std_logic;
					--sri_we 		: OUT std_logic;

					-- 2 i2s interfaces (common bclk and lrclk)
					bclk 			: in  std_logic;
					lrclk 		: in  std_logic;
					sdin1			: in  std_logic;
					sdout1		: out std_logic;
					sdin2			: in  std_logic;
					sdout2		: out std_logic
				);
end dap_core;

architecture Behavioral of dap_core is


---------------------------------------------
--
-- SDSP-Component (Filtering, Mixing and Voluming of Signals)
--
---------------------------------------------	

	COMPONENT sdsp_core_mc
	PORT(
		clk 			: IN 	std_logic;
		res 			: IN 	std_logic;
		input 		: IN 	signed(31 downto 0);
		opcode 		: IN 	std_logic_vector	( 3 downto 0);
		am_adr 		: IN 	std_logic_vector	( 8 downto 0);
		cm_adr 		: IN 	std_logic_vector	( 8 downto 0);
		cmi_adr 	: IN 	std_logic_vector	(10 downto 0);
		cmi_wdata : IN 	std_logic_vector	( 7 downto 0);
		cmi_wen 	: IN 	std_logic;
		cmi_en 		: IN 	std_logic;
	 	mmi_en		: in  std_logic;
		enable 		: IN 	std_logic;          
		output 		: OUT signed(31 downto 0);
		cmi_rdata : OUT std_logic_vector	( 7 downto 0);
	 	mmi_rdata	: out std_logic_vector	( 7 downto 0);
		ready 		: OUT std_logic
		);
	END COMPONENT;


---------------------------------------------
--
-- Utility-Core Component (Generating Audio-Signals)
--
---------------------------------------------	

	COMPONENT util_core
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		input : IN signed(31 downto 0);
		opcode : IN std_logic_vector(3 downto 0);
		am_adr : IN std_logic_vector(8 downto 0);
		cm_adr : IN std_logic_vector(8 downto 0);
		enable : IN std_logic;
		dmi_adr : IN std_logic_vector(10 downto 0);
		dmi_wdata : IN std_logic_vector(7 downto 0);
		dmi_wen : IN std_logic;
		dmi_en : IN std_logic;
		mmi_en : IN std_logic;
		wmi_en : IN std_logic;    
		--sri_data : INOUT std_logic_vector(15 downto 0);      
		output : OUT signed(31 downto 0);
		ready : OUT std_logic;
		--sri_addr : OUT std_logic_vector(17 downto 0);
		--sri_cs1 : OUT std_logic;
		--sri_cs2 : OUT std_logic;
		--sri_oe : OUT std_logic;
		--sri_ub : OUT std_logic;
		--sri_lb : OUT std_logic;
		--sri_we : OUT std_logic;
		dmi_rdata : OUT std_logic_vector(7 downto 0);
		mmi_rdata : OUT std_logic_vector(7 downto 0);
		wmi_rdata : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

---------------------------------------------
--
-- Program-Memory (PMEM) Compoment
--
---------------------------------------------	

	COMPONENT dap_pmem
	PORT(
		clk 				: IN 	std_logic;
		b_addr 			: IN 	std_logic_vector	(	8 downto 0);
		a_addr 			: IN 	std_logic_vector	(10 downto 0);
		a_wdata 		: IN 	std_logic_vector	(	7 downto 0);
		a_wen 			: IN 	std_logic;
		a_en 				: IN 	std_logic;      	    
		b_data 			: OUT std_logic_vector	(31 downto 0);
		a_rdata 		: OUT std_logic_vector	(	7 downto 0)
		);
	END COMPONENT;

---------------------------------------------
--
-- Memory-Compoment for SPI-Dispatcher
--
---------------------------------------------	

	COMPONENT device_6xbram_iomem
	PORT(
		clk 				: IN 	std_logic;
		res 				: IN 	std_logic;
		fsm_trg 		: IN 	std_logic;
		fsm_sel 		: IN 	std_logic;
		fsm_din 		: IN 	std_logic_vector	(	7 downto 0);
		bram_rdata1 : IN 	std_logic_vector	(	7 downto 0);
		bram_rdata2 : IN 	std_logic_vector	(	7 downto 0);
		bram_rdata3 : IN 	std_logic_vector	(	7 downto 0);
		bram_rdata4 : IN 	std_logic_vector	(	7 downto 0);
		bram_rdata5 : IN 	std_logic_vector	(	7 downto 0);
		bram_rdata6 : IN 	std_logic_vector	(	7 downto 0);
		iomem_rdata : IN 	std_logic_vector	(	7 downto 0);          
		fsm_rdy 		: OUT std_logic;
		fsm_dout 		: OUT std_logic_vector	(	7 downto 0);		 
		fsm_inf 		: OUT std_logic_vector	(15 downto 0);
		bram_adr 		: OUT std_logic_vector	(10 downto 0);
		bram_wdata 	: OUT std_logic_vector	(	7 downto 0);
		bram_wen 		: OUT std_logic;
		bram_en1 		: OUT std_logic;
		bram_en2 		: OUT std_logic;
		bram_en3 		: OUT std_logic;
		bram_en4 		: OUT std_logic;
		bram_en5 		: OUT std_logic;
		bram_en6 		: OUT std_logic;
		iomem_adr 	: OUT std_logic_vector	(	7 downto 0);
		iomem_wdata : OUT std_logic_vector	(	7 downto 0);
		iomem_wen 	: OUT std_logic;
		iomem_en 		: OUT std_logic
		);
	END COMPONENT;


---------------------------------------------
--
-- I2S-Compoment (single)
--
---------------------------------------------	

--	COMPONENT i2s_shifter_slave
--	PORT(
--		clk50m : IN std_logic;
--		bclk : IN std_logic;
--		lrclk : IN std_logic;
--		sdin : IN std_logic;
--		res : IN std_logic;
--		mute : IN std_logic;
--		sample_l_in : IN signed(23 downto 0);
--		sample_r_in : IN signed(23 downto 0);          
--		sdout : OUT std_logic;
--		new_sample : OUT std_logic;
--		sample_l_out : OUT signed(23 downto 0);
--		sample_r_out : OUT signed(23 downto 0)
--		);
--	END COMPONENT;

---------------------------------------------
--
-- I2S-Compoment (double)
--
---------------------------------------------	

	COMPONENT i2s_shifter_2x_slave
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		bclk : IN std_logic;
		lrclk : IN std_logic;
		sdin1 : IN std_logic;
		sdout1 : OUT std_logic;
		sdin2 : IN std_logic;
		sdout2 : OUT std_logic;
		new_sample : OUT std_logic;
		mute : IN std_logic;
		sample_1_in : IN signed(23 downto 0);
		sample_2_in : IN signed(23 downto 0);          
		sample_3_in : IN signed(23 downto 0);
		sample_4_in : IN signed(23 downto 0);          
		sample_1_out : OUT signed(23 downto 0);
		sample_2_out : OUT signed(23 downto 0);
		sample_3_out : OUT signed(23 downto 0);
		sample_4_out : OUT signed(23 downto 0)
		);
	END COMPONENT;

---------------------------------------------
--
-- Signals
--
---------------------------------------------	

	--
	-- Input and Output Register and signals
	--

	-- enable signal for storing output Signals from SDP/UTIL-Core
	signal ena_route		: std_logic;

	-- temporary output-signals (to i2s-shifter, after new_sample)
	signal toutput1  		: signed (31 downto 0);
	signal toutput2  		: signed (31 downto 0);
	signal toutput3  		: signed (31 downto 0);
	signal toutput4  		: signed (31 downto 0);

	-- output-signals from SDSP and UTIL-Core for feedback
	signal sdsp_output_reg  : signed (31 downto 0);
	signal util_output_reg  : signed (31 downto 0);

	-- input-signals from I2S interface 
	signal core_input  	: signed (31 downto 0);
	signal sdsp_output 	: signed (31 downto 0);
	signal util_output 	: signed (31 downto 0);

	-- SDSP/UTIL-Core trigger signals
	signal bq_trg				: std_logic;
	signal bq_rdy				: std_logic;

	-- SDSP/UTIL-Core ready signals
	signal ut_trg				: std_logic;
	signal ut_rdy				: std_logic;
	
	--
	-- program-statemachine signale
	--

	-- Program-State-Signal for Main Statemachine
	signal ps_state   	: std_logic_vector ( 3 downto 0);

	-- Program-State-Machine Adress and Data from BlockRAM PMEM
	signal ps_adr     	: std_logic_vector ( 8 downto 0);
	signal ps_opcode  	: std_logic_vector (31 downto 0);

	-- Signals splitted up from PS_OPCODE
	signal opc_bq_opc 	: std_logic_vector ( 3 downto 0);
	signal opc_am_adr		: std_logic_vector ( 8 downto 0);
	signal opc_cm_adr		: std_logic_vector ( 8 downto 0);
	signal opc_in_sel  	: std_logic_vector ( 3 downto 0);
	signal opc_out_sel 	: std_logic_vector ( 3 downto 0);
	signal opc_core_sel	: std_logic;
	signal opc_eop			: std_logic;

	--
	-- Memory Interfaces (from DEVICE_6xBRAM_IOMEM)
	--

	-- Common BlockRAM Signals for all connected Block-RAMs
	signal cmi_adr	  	: std_logic_vector (10 downto 0); 
	signal cmi_wdata		: std_logic_vector ( 7 downto 0);
	signal cmi_wen			: std_logic;


	-- Data and Control for Coeffcient-BlockRAM CMEM
	signal cmi_rdata 		: std_logic_vector ( 7 downto 0);
	signal cmi_en 			: std_logic;

	-- Data and Control for Program-BlockRAM PMEM
	signal pmi_rdata 		: std_logic_vector ( 7 downto 0);
	signal pmi_en 			: std_logic;

	-- Data and Control for SDSP-MicroCode-BlockRAM MMEM
	signal mmi_rdata 		: std_logic_vector ( 7 downto 0);
	signal mmi_en 			: std_logic;

	-- Data and Control for UtilCore-Data-BlockRAM DMEM 
	signal dmi_rdata 		: std_logic_vector ( 7 downto 0);
	signal dmi_en 			: std_logic;

	-- Data and Control for UtilCore-Wavetable-BlockRAM CMEM (unconnected)
	signal wmi_rdata 		: std_logic_vector ( 7 downto 0);
	signal wmi_en 			: std_logic;

	-- Data and Control for UtilCore-MicroCode-BlockRAM UMEM (unconnected)
	signal umi_rdata 		: std_logic_vector ( 7 downto 0);
	signal umi_en 			: std_logic;



	constant c_bq_sel : std_logic := '0';
	constant c_ut_sel : std_logic := '1';


	--
	-- Memory Mapped IO (from DEVICE_6xBRAM_IOMEM)
	--

	-- Signals for Memory-Mapped IO
	signal reg_adr 		: std_logic_vector ( 7 downto 0); 
	signal reg_wdata	: std_logic_vector ( 7 downto 0);
	signal reg_rdata	: std_logic_vector ( 7 downto 0);
	signal reg_wen		:	std_logic;
	signal reg_en			:	std_logic;

	-- register for read-only statistics (Frame-Length, Program-Length in Cycles)
	-- internal counter
  signal cnt_sampleperiod : std_logic_vector (15 downto 0);
	signal cnt_dspactive	  : std_logic_vector (15 downto 0);
	
	-- resulting output
	signal reg_sampleperiod : std_logic_vector (15 downto 0);
	signal reg_dspactive	  : std_logic_vector (15 downto 0);

	-- Signals generated by Memory Mapped IO (currently only Mute)
	signal mute_out  				: std_logic;




	-- Edge detect for NewSample (from I2S_SHIFTER_2X_SLAVE)
 	signal new_smp_z1				: std_logic;
 	signal new_smp  				: std_logic;

	-- input & output signals for I2S Interfaces
	-- must be signed to have correct type at SDSP/UTIL-Core IOs
	signal input_1  : signed (23 downto 0);
	signal input_2  : signed (23 downto 0);
	signal input_3  : signed (23 downto 0);
	signal input_4  : signed (23 downto 0);
	signal output_1 : signed (23 downto 0);
	signal output_2 : signed (23 downto 0);
	signal output_3 : signed (23 downto 0);
	signal output_4 : signed (23 downto 0);

	

begin

---------------------------------------------
--
-- Instanciation of components
--
---------------------------------------------	

	--
	-- Instance of SDSP-Core
	--		

	Inst_sdsp_core_mc : sdsp_core_mc PORT MAP(
		clk 			=> clk,
		res 			=> res,

		-- Input and Output
		input 		=> core_input,
		output 		=> sdsp_output,

		-- Opcode to be executed (with parameters AMEM_ADR and CMEM_ADR)
		opcode 		=> opc_bq_opc,
		am_adr 		=> opc_am_adr,
		cm_adr 		=> opc_cm_adr,
		enable 		=> bq_trg,
		ready 		=> bq_rdy,

		-- memmory Interface for controlling SDSP's Operation
		-- common signals
		cmi_adr 	=> cmi_adr,
		cmi_wdata => cmi_wdata,
		cmi_wen 	=> cmi_wen,
		-- CMEM-BlockRAM signals
		cmi_rdata => cmi_rdata,
		cmi_en 		=> cmi_en,
		-- MMEM-BlockRAM signals
		mmi_rdata	=> mmi_rdata,
		mmi_en		=> mmi_en		
	);

	--
	-- Instance of UTIL-Core 
	--		

  Inst_util_core: util_core PORT MAP(
		clk 			=> clk,
		res 			=> res,
		-- Input and Output
		input 		=> core_input,
		output 		=> util_output,

	-- Opcode to be executed (with parameters AMEM_ADR and CMEM_ADR)
		opcode 		=> opc_bq_opc,
		am_adr 		=> opc_am_adr,
		cm_adr 		=> opc_cm_adr,
		enable 		=> ut_trg,
		ready 		=> ut_rdy,

		-- SRAM-Access 
		--sri_data 	=> sri_data,
		--sri_addr 	=> sri_addr,
		--sri_cs1 	=> sri_cs1,
		--sri_cs2 	=> sri_cs2,
		--sri_oe 		=> sri_oe,
		--sri_ub 		=> sri_ub,
		--sri_lb 		=> sri_lb,
		--sri_we 		=> sri_we,

		-- memmory Interface for controlling Util-Core's Operation
		-- common signals
  	dmi_adr 	=> cmi_adr,
		dmi_wdata => cmi_wdata,
		dmi_wen 	=> cmi_wen,
 		-- DMEM-BlockRAM signals
		dmi_rdata => dmi_rdata,
		dmi_en 		=> dmi_en,
		-- WMEM-BlockRAM signals
		wmi_rdata => wmi_rdata,
		wmi_en 		=> wmi_en,
		-- UMEM-BlockRAM signals
		mmi_rdata => umi_rdata,
		mmi_en 		=> umi_en
	);

	--
	-- Instance of Program-Memory of DAP
	--		

	Inst_dap_pmem: dap_pmem PORT MAP(
		clk 		=> clk,
		-- DAP-Side-Interface
		b_addr 	=> ps_adr,
		b_data 	=> ps_opcode,
		-- SPI-Dispatcher Side Interface
		a_addr 	=> cmi_adr,
		a_wdata => cmi_wdata,
		a_wen 	=> cmi_wen,
		a_rdata => pmi_rdata,
		a_en 		=> pmi_en
	);


	--
	-- Device for accessing 6 BlockRAMs and MemoryMapped IO
	--		

	Inst_device_6xbram_iomem: device_6xbram_iomem PORT MAP(
		clk 				=> clk,
		res 				=> res,
		
		--SPI Dipatcher Signals
		fsm_trg 		=> fsm_trg,
		fsm_rdy 		=> fsm_rdy,
		fsm_sel 		=> fsm_sel,
		fsm_din 		=> fsm_din,
		fsm_dout 		=> fsm_dout,
		fsm_inf 		=> open,

		-- Common BlockRAM Signals
		bram_adr 		=> cmi_adr,
		bram_wdata 	=> cmi_wdata,
		bram_wen 		=> cmi_wen,

		-- BlockRAM 1 Signals	(-> PMEM)	(0x0000 - 0x1fff)
		bram_rdata1 => (others => '0'),
		--bram_rdata1 => pmi_rdata,
		bram_en1 		=> pmi_en,

		-- BlockRAM 2 Signals (-> CMEM)	(0x2000 - 0x3fff)
		bram_rdata2 => (others => '0'),
		--bram_rdata2 => cmi_rdata,
		bram_en2	 	=> cmi_en,

		-- BlockRAM 3 Signals (-> MMEM) (0x4000 - 0x5fff)
		bram_rdata3 => (others => '0'),
		--bram_rdata3 => mmi_rdata,
		bram_en3 		=> mmi_en,

		-- BlockRAM 4 Signals (-> DMEM) (0x6000 - 0x7fff)
		bram_rdata4 => (others => '0'),
		--bram_rdata4 => dmi_rdata,
		bram_en4 		=> dmi_en,

		-- BlockRAM 5 Signals	(-> WMEM) (0x8000 - 0x9fff)
		bram_rdata5 => (others => '0'),
		--bram_rdata5 => wmi_rdata,
		bram_en5 		=> wmi_en,

		-- BlockRAM 6 Signals (-> UMEM) (0xa000 - 0xbfff)
		bram_rdata6 => (others => '0'),
		--bram_rdata6 => umi_rdata,
		bram_en6 		=> umi_en,

		-- Memory Mapped IO Signals
		iomem_adr 	=> reg_adr,
		iomem_wdata => reg_wdata,
		iomem_rdata => (others => '0'),
		--iomem_rdata => reg_rdata,
		iomem_wen 	=> reg_wen,
		iomem_en 		=> reg_en
	);


	--
	-- Instance of Single I2S Interface
	--		

--	Inst_i2s_shifter_slave: i2s_shifter_slave PORT MAP(
--		clk50m 				=> clk,
--		res 					=> res,
--
--		-- IO-Connections for I2S Interface
--		bclk 					=> bclk,
--		lrclk 				=> lrclk,
--		sdin 					=> sdin,
--		sdout 				=> sdout,
--
--		-- Main Control and Receive Signals
--		mute 					=> mute_out,
--		--mute 					=> '0',
--		new_sample 		=> new_smp,
--
--		-- 2 Inputs
--		sample_l_in 	=> signed (output_l),
--		sample_r_in 	=> signed (output_r),
--
--		-- 2 Outputs
--		sample_l_out 	=> input_l,
--		sample_r_out 	=> input_r
--	);


	--
	-- Instance of Dual I2S Interface
	--		

	Inst_i2s_shifter_2x_slave: i2s_shifter_2x_slave PORT MAP(
		clk50m 				=> clk,
		res 					=> res,

		-- IO-Connections for I2S Interface
		bclk 					=> bclk,
		lrclk 				=> lrclk,
		sdin1					=> sdin1,
		sdout1				=> sdout1,
		sdin2					=> sdin2,
		sdout2				=> sdout2,

		-- Main Control and Receive Signals
		mute 					=> mute_out,
		--mute 					=> '0',
		new_sample 		=> new_smp,

		-- 4 Inputs
		sample_1_in 	=> signed (output_1),
		sample_2_in 	=> signed (output_2),
		sample_3_in 	=> signed (output_3),
		sample_4_in 	=> signed (output_4),

		-- 4 Outputs
		sample_1_out 	=> input_1,
		sample_2_out 	=> input_2,
		sample_3_out 	=> input_3,
		sample_4_out 	=> input_4
	);


---------------------------------------------
--
-- Processes and Logic
--
---------------------------------------------	




	--
	-- input and output routing (within DAP-Core)
	--
	process (clk, res)
	begin
	  if res = '1' then
		  toutput1 				<= (others => '0'); 
		  toutput2 				<= (others => '0'); 
		  toutput3 				<= (others => '0'); 
		  toutput4 				<= (others => '0'); 
			sdsp_output_reg <= (others => '0'); 
			util_output_reg <= (others => '0'); 
		  core_input 			<= (others => '0'); 

		elsif clk'event and clk = '1' then

			-- Input-Muxer for SDSP/Util-Core
			case opc_in_sel is
				when X"0" 	=> core_input <= signed  (input_1 (23) & input_1 (23) & input_1 (23) & input_1 (23) & 
																					 		input_1 (23) & input_1 (23) & input_1 (23) & input_1 (23) & input_1 (23 downto 0));
				when X"1" 	=> core_input <= signed  (input_2 (23) & input_2 (23) & input_2 (23) & input_2 (23) & 
																	 				 		input_2 (23) & input_2 (23) & input_2 (23) & input_2 (23) & input_2 (23 downto 0));
				when X"2" 	=> core_input <= signed  (input_3 (23) & input_3 (23) & input_3 (23) & input_3 (23) & 
																					 		input_3 (23) & input_3 (23) & input_3 (23) & input_3 (23) & input_3 (23 downto 0));
				when X"3" 	=> core_input <= signed  (input_4 (23) & input_4 (23) & input_4 (23) & input_4 (23) & 
																	 				 		input_4 (23) & input_4 (23) & input_4 (23) & input_4 (23) & input_4 (23 downto 0));
				when X"e" 	=> core_input <= util_output_reg;
				when X"f" 	=> core_input <= sdsp_output_reg;
				when others => NULL;
			end case;

			-- Output-Muxer for SDSP/Util-Core
		  if ena_route = '1' then

				-- SDSP/Util-Core Output Register for internal Feedback
				sdsp_output_reg <= sdsp_output;
				util_output_reg <= util_output;

				-- Output Routing from SDSP or Util-Core into temporarily Register
				case opc_out_sel is
					when X"0" 	=> toutput1 <= sdsp_output;
					when X"1" 	=> toutput2 <= sdsp_output;
					when X"2" 	=> toutput3 <= sdsp_output;
					when X"3" 	=> toutput4 <= sdsp_output;
					when X"4" 	=> toutput1 <= util_output;
					when X"5" 	=> toutput2 <= util_output;
					when X"6" 	=> toutput3 <= util_output;
					when X"7" 	=> toutput4 <= util_output;
					when others => NULL;
				end case;
			end if;
		end if;
	end process;


	-- Mapping PMEM Opcode 

	opc_bq_opc 		<= ps_opcode (31 downto 28);				-- opcode for SDSP/Util-Core (simultaneous)

	opc_core_sel	<= ps_opcode (          25);				-- Core Select (0 = SDSP / 1 = Util)
	opc_eop				<= ps_opcode (          24);				-- end of program	

	opc_in_sel		<= ps_opcode (23 downto 20); 				-- Input-Select  (-> SDSP and Util_core)
	opc_out_sel		<= ps_opcode (19 downto 16); 				-- Output-Select (-> SDSP and Util_core)


	opc_am_adr 		<= '0' & ps_opcode (15 downto  8); 	-- AMEM-Addr for SDSP and Util-Core (8 bit only !!)
	opc_cm_adr 		<= '0' & ps_opcode ( 7 downto  0); 	-- CMEM-Addr for SDSP and Util-Core (8 bit only !!)



	-- PMEM-Opcode Statemachine
	process (clk, res)
	begin
	  if res = '1' then
			ps_state		<= (others => '0');	
			ps_adr			<= (others => '0');
			bq_trg			<= '0';
			ena_route		<= '0';
			output_1			<= (others => '0');
			output_2			<= (others => '0');
			output_3			<= (others => '0');
			output_4			<= (others => '0');
		
			reg_rdata   <= (others => '0');
			mute_out		<= '0';

			new_smp_z1  <= '0';

			cnt_sampleperiod <= (others => '0');
		  cnt_dspactive	 	 <= (others => '0');

			reg_sampleperiod <= (others => '0');
			reg_dspactive	   <= (others => '0');

		elsif	clk'event and clk = '1' then

			-- Statemachine :
			-- State 0 			: Wait for New Sample (new_sample)	and Reset Program Counter (ps_adr)
			-- State 1 			: Wait for Current Opcode and "Decode"
			-- State 2 			: Check if Program end or Execute Opcode
			-- State 3 & 4 	: Wait Cycles 
			-- State 5			: Wait for Ready and Increment Program Counter
			case ps_state is
			  -- Wait for New_Sample
			  when X"0" => if new_smp = '1' then
											 ps_state 	<= X"1";
											 -- reset program counter
											 ps_adr  		<= (others => '0');

											 -- temporary output -> output-registers
  										 output_1 		<= toutput1 (23 downto 0);
											 output_2 		<= toutput2 (23 downto 0);
  										 output_3 		<= toutput3 (23 downto 0);
											 output_4 		<= toutput4 (23 downto 0);
										 	 ena_route 	<= '0';
										 end if;
				-- wait for opcode and "decode" it
				when X"1" => ps_state 	<= X"2";
										 ena_route 	<= '0';

				-- execute opode
				when X"2" => -- check for program-end
										 --if (opc_eop = '0') or (ps_adr = '1' & X"ff") then
										 if (opc_eop = '0') then
										   -- if not ...
											 if opc_core_sel = c_bq_sel then
					               bq_trg 	<= '1';			  -- trigger SDSP-Core
												 ps_state <= X"3";
											 else
					               ut_trg 	<= '1';			  -- trigger Util-Core
												 ps_state <= X"3";
											 end if;
										 else
										  -- if so ...
				               bq_trg 	<= '0';				-- end program (-> state 0)
											 ps_state <= X"0";
										 end if;


				-- wait 2 cycles 
				when X"3" => ps_state <= X"4";
										 bq_trg 	<= '0';
										 ut_trg 	<= '0';
				when X"4" => ps_state <= X"5";
										 bq_trg 	<= '0';
										 ut_trg 	<= '0';

				-- wait for ready and increment adr-counter 
				when X"5" => 	-- which core was it ?!
											if (bq_rdy = '1' and opc_core_sel = c_bq_sel) or (ut_rdy = '1' and opc_core_sel = c_ut_sel) then
--											if opc_core_sel = '0' then
												-- common : increment opcode if ready and take over core-output registers into temp. outputs
												
												-- wait for ready from SDSP-Core
											 	if (bq_rdy = '1') then
												 	ps_adr 		<= ps_adr + 1;
												 	ps_state 	<= X"1";
											   	ena_route <= '1';	
												-- or wait for ready from Util-Core
												elsif (ut_rdy = '1') then
												 	ps_adr 		<= ps_adr + 1;
												 	ps_state 	<= X"1";
												  ena_route <= '1';	
												end if;
										 	end if;
				when others => NULL;
			end case;

			-- memory-mapped io functions

			if reg_en = '1' then
			  -- read io
				if reg_wen = '0' then
					case reg_adr (3 downto 0) is
					 	-- register 0 : hi-byte from frame-length-register
					  when X"0" 		=> reg_rdata <= reg_sampleperiod (15 downto 8);
						-- register 1 : lo-byte from frame-length-register
					  when X"1" 		=> reg_rdata <= reg_sampleperiod ( 7 downto 0);
						-- register 2 : hi-byte from active-length-register
					  when X"2" 		=> reg_rdata <= reg_dspactive    (15 downto 8);
						-- register 3 : hi-byte from active-length-register
					  when X"3" 		=> reg_rdata <= reg_dspactive    ( 7 downto 0);
					  when others   => reg_rdata <= X"00";
					end case;
				-- write io
				else
					case reg_adr (3 downto 0) is
					  -- register 0 : bit 0 -> mute i2s-interface on/off
					  when X"0" 		=> mute_out <= reg_wdata (0);
						when others   => NULL;
					end case;
				end if;
			end if;

			-- edge detect for new-sample-signal
			new_smp_z1 <= new_smp;

			-- until we have no new sample -> count cycles
			if new_smp_z1 = '0' and new_smp = '0' then
			  cnt_sampleperiod <= cnt_sampleperiod + '1';
				-- to determine active-length we have to count only when not waiting for a new sample
				if ps_state /= "0" then
					cnt_dspactive 	<= cnt_dspactive + '1';
				end if;
			-- if we have a new sample, we take over our counter values into output-registers for io-read
  		elsif new_smp_z1 = '0' and new_smp = '1' then 
				cnt_sampleperiod  <= (others => '0');			 
				cnt_dspactive 		<= (others => '0');
				reg_sampleperiod 	<= cnt_sampleperiod;
				reg_dspactive 		<= cnt_dspactive;
			end if;

		end if;

		-- SPI-Dispatcher Device Info 
		fsm_inf <= X"3300";

	end process;

end Behavioral;
