-----------------------------------------------------------------------------
--
--	CLOCKGEN.VHDL
--
--	Simple Clock-Generation (50 MHz -> 1MHz -> 10kHz -> 1kHz -> 100Hz -> 1Hz)
--
--	Author 	: Rene Bllhoff
--  Created : around May 2005
--
--	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
-----------------------------------------------------------------------------
--
--  This source file may be used and distributed without
--  restriction provided that this copyright statement is not
--  removed from the file and that any derivative work contains
--  the original copyright notice and the associated disclaimer.
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--										
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public
--  License along with this source; if not, download it	from
--  http://www.gnu.org/licenses/gpl.html	
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity clockgen is
   port (
		 clk50m  	: in  std_logic;
		 res			: in 	std_logic;
		 clk1mo		: out std_logic;
		 clk10ko	: out std_logic;
		 clk1ko		: out std_logic;
		 clk100o	: out std_logic;
		 clk1o		: out std_logic
	 );
end clockgen;

architecture Behavioral of clockgen is

	signal cnt50m : std_logic_vector (5 downto 0); -- / 50   	-> 6 bit
	signal cnt10k : std_logic_vector (6 downto 0); -- / 100		-> 7 bit
	signal cnt1k  : std_logic_vector (3 downto 0); -- / 10		-> 4 bit
	signal cnt100 : std_logic_vector (3 downto 0); -- / 10 		-> 4 bit
	signal cnt1   : std_logic_vector (6 downto 0); -- / 100		-> 6 bit

	signal clk1m 	: std_logic;
	signal clk10k	: std_logic;
	signal clk1k 	: std_logic;
	signal clk100	: std_logic;
	signal clk1  	: std_logic;

begin

	-- / 50	-> 1mhz
	process (clk50m, res)
	begin
		if res = '1' then
		  cnt50m 	<= (others => '0');
			clk1m 	<= '0';
		elsif clk50m'event and clk50m = '1' then
			cnt50m <= cnt50m + '1';
			if cnt50m = "110010" then
				cnt50m 	<= (others => '0');
				clk1m 	<= '1';
			else
				clk1m   <= '0';
			end if;
		end if;
	end process;

	-- / 100 -> 10kHz

	process (clk50m, res)
	begin
		if res = '1' then
		  cnt10k 	<= (others => '0');
			clk10k 	<= '0';
		elsif clk50m'event and clk50m = '1' then
		  if clk1m = '1' then
				cnt10k <= cnt10k + '1';
				if cnt10k = "1100100" then
					cnt10k 	<= (others => '0');
					clk10k 	<= '1';
				else
					clk10k  <= '0';
				end if;
      else
				clk10k   	<= '0';
			end if;
		end if;
	end process;

	-- / 10 -> 1kHz

	process (clk50m, res)
	begin
		if res = '1' then
		  cnt1k 	<= (others => '0');
			clk1k 	<= '0';
		elsif clk50m'event and clk50m = '1' then
		  if clk10k = '1' then
				cnt1k <= cnt1k + '1';
				if cnt1k = "1010" then
					cnt1k 	<= (others => '0');
					clk1k 	<= '1';
				else
					clk1k   <= '0';
				end if;
			else
			  clk1k 		<= '0';
			end if;
		end if;
	end process;

	-- /  10 -> 100 Hz

	process (clk50m, res)
	begin
		if res = '1' then
		  cnt100 	<= (others => '0');
			clk100 	<= '0';
		elsif clk50m'event and clk50m = '1' then
		  if clk1k = '1' then
				cnt100 <= cnt100 + '1';
				if cnt100 = "1010" then
					cnt100 	<= (others => '0');
					clk100 	<= '1';
				else
					clk100   <= '0';
				end if;
			else
				clk100   	<= '0';
			end if;
		end if;
	end process;

	-- / 100 -> 1Hz

	process (clk50m, res)
	begin
		if res = '1' then
		  cnt1 	<= (others => '0');
			clk1 	<= '0';
		elsif clk50m'event and clk50m = '1' then
		  if clk100 = '1' then
				cnt1 <= cnt1 + '1';
				if cnt1 = "1100100" then
					cnt1 	<= (others => '0');
					clk1 	<= '1';
				else
					clk1  <= '0';
				end if;
			else
				clk1  	<= '0';
			end if;
		end if;
	end process;

	clk1mo 	<= clk1m;
	clk10ko <= clk10k;
	clk1ko 	<= clk1k;
	clk100o <= clk100;
	clk1o 	<= clk1;

end Behavioral;
